//-----------------------------------------------------------------------------
//
//  Audio.h++
//
//  Audio Input and Output Class Declaration
//
//  Tony Jebara (Copyright (C) 1997)
//  Brian Clarkson
//
//-----------------------------------------------------------------------------

#ifndef Audio_h
#define Audio_h

// The LIFO buffer
#include "CircularBuffer.h++"

// System includes
#include <errno.h>
#include <dmedia/audio.h>
#include <stdlib.h>
#include <stdio.h>

// Default sizes
#define INPUT_Q_SIZE 8000
#define SAMPLE_RATE 16000

class AudioIn : public CircularFloatBuffer {
  // Construction/Destruction
public:
  AudioIn();
  ~AudioIn();

  //Data
private:
  // Audio configuration
  int mic;
  int speaker;
  ALport inputPort;
  ALport outputPort;
  ALconfig audioConfigIn;
  ALconfig audioConfigOut;
  ALpv pvs[2];
  // State
  int IsPortOpen;
  // Channel characterstics
  float mean;
  float min, max;
  // Q Size
  int qsize;

public:

  //Methods
private:

public:
  void   MinMax();
  void   Open();
  void   Close();
  int    Read(void);
  void   Read(int len);
  int    Write(int len, float *buffer);
  int    Write(int len, float *buffer, int startfrom);
  void   SetQSize(int s);
};

#endif

