//-----------------------------------------------------------------------------
//
//  CircularBuffer.c++
//
//  Circular Buffer Class Implementation
//
//  Tony Jebara (Copyright (C) 1997)
//  Brian Clarkson
//
//-----------------------------------------------------------------------------

#include "CircularBuffer.h++"

template<class T> CircularBuffer<T>::CircularBuffer() {
  size = DEFAULT_CIRCULAR_BUFFER_SIZE;
  data = new T[size];
  pos = 0;
  tmp = NULL;
}

template<class T> CircularBuffer<T>::CircularBuffer(int n) {
  size = n;
  data = new T[size];
  pos = 0;
  tmp = NULL;
}

template<class T> CircularBuffer<T>::~CircularBuffer() {
  delete [] data;
  if(tmp)
    delete [] tmp;
}

template<class T> void CircularBuffer<T>::Reset() {
  delete [] data;
  if(tmp)
    delete [] tmp;
  size = DEFAULT_CIRCULAR_BUFFER_SIZE;
  data = new T[size];
  pos = 0;
}

template<class T> void CircularBuffer<T>::Reset(int n) {
  delete [] data;
  if(tmp)
    delete [] tmp;
  tmp = NULL;
  size = n;
  data = new T[size];
  pos = 0;
}

template<class T> void CircularBuffer<T>::Add(T * newdata, int len) {
  int n;

  // If we are asked to add more data than we have space
  // take a shortcut and just add the last part
  if(size < len) {
    pos = 0;
    Add(&(newdata[len - size]),size);
    return;
  }
  // Detect wrap around in the circular buffer
  n = size - pos; // space left
  if(n < len) {
    // Read what we can
    if(n + pos > size)
      fprintf(stderr,"Out of bounds in circular buffer\n");
    memcpy(&(data[pos]),newdata,sizeof(T)*n);
    pos = 0;
    // Then wrap around and read the rest 
    // We know that len - n <= size since len <= size
    if(len - n + pos > size)
      fprintf(stderr,"Out of bounds in circular buffer\n");
    memcpy(&(data[pos]),&(newdata[n]),sizeof(T)*(len - n));
  } else { 
    // Read all the data
    if(len + pos > size)
      fprintf(stderr,"Out of bounds in circular buffer\n");
    memcpy(&(data[pos]),newdata,sizeof(T)*len);
    // Increment the circular buffer position
    pos += len;
  }
}

template<class T> T * CircularBuffer<T>::GetLast(int len) {
  int n;

  if(len > size) {
    fprintf(stderr,"CircularBuffer::GetLast - you asked for too much data (%d > %d)\n",
	   len,size);
    exit(1);
  }

  // Detect wrap around in the circular buffer
  n = pos - len; // the beginning of the data
  if(n < 0) {
    if(tmp)
      delete [] tmp;
    tmp = new T[len];
    // Need to wrap around and grab data from the end
    memcpy(tmp,&(data[size + n]),sizeof(T)*(-n));
    // Now copy the rest of the data, located at the front
    memcpy(&(tmp[-n]),data,sizeof(T)*pos);
    // Return the pointer to tmp
    return tmp;
  } 
  
  // We can just return a pointer offset len backwards
  return &(data[n]);

}




